#include "bmp_utils.h"

enum read_status read_from_bmp(FILE *file, struct image *img) {
    struct bmp_header header;
    if (!fread(&header, sizeof(struct bmp_header), 1, file)) {
        return READ_INVALID_HEADER;
    } if (header.bfType != 0x4D42) {
        return READ_INVALID_SIGNATURE;
    } if (header.biBitCount != 24) {
        return READ_INVALID_BITS;
    }
    img->width = header.biWidth;
    img->height = header.biHeight;
    const uint8_t symbols = (BMP_PADDING_MULTIPLE - img->width * sizeof(struct pixel) % BMP_PADDING_MULTIPLE) % BMP_PADDING_MULTIPLE;
    img->data = malloc(img->width * img->height * sizeof(struct pixel) + img->height * symbols);
    if (img->data == NULL) {
        return READ_MEMORY_ERROR;
    }
    for (long y = 0; y < img->height; y++) {
        if (!fread(img->data + y * img->width, sizeof(struct pixel), img->width, file)) {
            free(img->data);
            return READ_MEMORY_ERROR;
        }
        fseek(file, symbols, SEEK_CUR);
    }
    return READ_OK;
}


enum write_status write_to_bmp(FILE *file, struct image *img) {
    struct bmp_header header = {0};
    const uint8_t symbols = (BMP_PADDING_MULTIPLE - img->width * sizeof(struct pixel) % BMP_PADDING_MULTIPLE) % BMP_PADDING_MULTIPLE;
    header.bfType = BMP_SIGNATURE;
    header.bfileSize = sizeof(header) + img->height * (img->width + symbols) + sizeof(header.bfOffBits);
    header.bfOffBits = sizeof(header);
    header.biWidth = img->width;
    header.biHeight = img->height;
    header.biSize = BI_SIZE;
    header.biPlanes = BI_PLANES;
    header.biBitCount = BMP_BITS_PER_PIXEL;
    header.biCompression = 0;
    header.biSizeImage = img->height * (img->width + symbols);
    fwrite(&header, sizeof(header), 1, file);

    uint8_t padding[3] = {0};
    for (size_t y = 0; y < img->height; y++) {
        if (!fwrite(img->data + y * img->width, sizeof(struct pixel), img->width, file)) {
            return WRITE_ERROR;
        }
        fwrite(padding, symbols, 1, file);
    }
    return WRITE_OK;
}
