#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "../include/bmp.h"
#include "../include/bmp_utils.h"
#include "../include/img_utils.h"
#include "../include/transformation_utils.h"


int main(const int argc, char *argv[]) {
    if (argc != 4){
        fprintf(stderr, "ERROR: not enough arguments!\n");
        return 1;
    }
    // Чтение файла
    FILE *file_for_reading = fopen(argv[1], "rb");
    if (file_for_reading == NULL) {
        return errno;
    }

    struct image image = {0};
    const enum read_status status_of_reading = read_from_bmp(file_for_reading, &image);
    if (status_of_reading != READ_OK) {
        fprintf(stderr, "ERROR: %d\n", status_of_reading);
        return errno;
    }

    // манипуляции с картинкой
    struct image new_image = {0};
    char *command = argv[3];
    if (strcmp(command, "cw90") == 0) {
        new_image = rotate_90_cw(&image);
    } else if (strcmp(command, "ccw90") == 0) {
        new_image = rotate_90_ccw(&image);
    } else if (strcmp(command, "fliph") == 0) {
        new_image = flip_horizontal(&image);
    } else if (strcmp(command, "flipv") == 0) {
        new_image = flip_vertical(&image);
    } else if (strcmp(command, "none") == 0) {
        new_image = copy_image(&image);
    } else {
        fprintf(stderr, "ERROR: transformation not found!\n");
        return 1;
    }

    // сохраняем картинку в файлик
    FILE *file_for_writing = fopen(argv[2], "wb");
    const enum write_status status_of_writing = write_to_bmp(file_for_writing, &new_image);
    if (status_of_writing != WRITE_OK) {
        fprintf(stderr, "ERROR: %d\n", status_of_writing);
        return errno;
    }

    free_image(&new_image);
    fclose(file_for_reading);
    fclose(file_for_writing);
    return 0;
}
