#include "transformation_utils.h"

struct image rotate_90_ccw(struct image *img) {
    const struct image new_img = create_image(img->width, img->height);
    size_t old_index = 0;
    size_t new_index = 0;
    for (size_t y = 0; y < img->height; y++) {
        for (size_t x = 0; x < img->width; x++) {
            old_index = y * img->width + x;
            new_index = (x + 1) * new_img.width - y - 1;
            new_img.data[new_index] = img->data[old_index];
        }
    }
    free_image(img);
    return new_img;
}

struct image rotate_90_cw(struct image *img) {
    struct image rotated = rotate_90_ccw(img);
    rotated = rotate_90_ccw(&rotated);
    rotated = rotate_90_ccw(&rotated);
    free_image(img);
    return rotated;
}

struct image flip_horizontal(struct image *img) {
    const struct image new_img = create_image(img->height, img->width);
    size_t old_index = 0;
    size_t new_index = 0;
    for (size_t y = 0; y < img->height; y++) {
        for (size_t x = 0; x < img->width; x++) {
            old_index = y * img->width + x;
            new_index = (y + 1) * img->width - x - 1;
            new_img.data[new_index] = img->data[old_index];
        }
    }
    free_image(img);
    return new_img;
}

struct image flip_vertical(struct image *img) {
    const struct image new_img = create_image(img->height, img->width);
    size_t old_index = 0;
    size_t new_index = 0;
    for (size_t y = 0; y < img->height; y++) {
        for (size_t x = 0; x < img->width; x++) {
            old_index = y * img->width + x;
            new_index = img->width * img->height - img->width * (y + 1) + x;
            new_img.data[new_index] = img->data[old_index];
        }
    }
    free_image(img);
    return new_img;
}

struct image copy_image(struct image *img) {
    const struct image new_img = create_image(img->height, img->width);
    size_t index = 0;
    for (size_t y = 0; y < img->height; y++) {
        for (size_t x = 0; x < img->width; x++) {
            index = y * img->width + x;
            new_img.data[index] = img->data[index];
        }
    }
    free_image(img);
    return new_img;
}
